#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include <signal.h>
#include "currentValue.h"
#include "nextStep.h"

volatile sig_atomic_t keep_running = 0;
extern int debug_level;

RETSIGTYPE stop_server( int a ) {
   if( debug_level != 0 )
      snmp_log( LOG_INFO, "\n--------- got signal: %d -------------\n", a );
   keep_running = 0;
}

#define daemon_title "myagn" // имя демона и конфиг-файлов!

int main ( int argc, char **argv ) {
   int c,
       agentx_subagent = 0;  // 0 - SNMP master agent : 1 - AgentX subagent
   while( -1 != ( c = getopt( argc, argv, "xv" ) ) )
      switch( c ) {
         case 'x': agentx_subagent = 1; break;
         case 'v': debug_level++; break;
         default : 
            exit( EXIT_FAILURE );
      }


init_snmp_logging();
snmp_enable_syslog();

// snmp_enable_calllog();
   snmp_enable_stderrlog();      // print log errors to stderr 
// if( background && netsnmp_daemonize( 1, !syslog ) ) exit( 1 );
   if( debug_level > 1 )snmp_set_do_debugging( 1 );
   if( agentx_subagent )         // we're an agentx subagent? 
      netsnmp_ds_set_boolean( NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_ROLE, 1 );
   SOCK_STARTUP;                 // initialize tcpip, if necessary
   init_agent( daemon_title );   // initialize the agent library
   init_currentValue();          // initialize mib code here...
   init_nextStep();
   init_snmp( daemon_title );    // имя конфиг-файла
   if( !agentx_subagent )        // If we're going to be a snmp master agent, initial the ports 
      init_master_agent();       // open the port to listen on (defaults to udp:161) 
   signal( SIGTERM, stop_server );
   signal( SIGINT, stop_server );
   if( debug_level != 0 ) snmp_log( LOG_INFO, "%s is up and running.\n", daemon_title );
   keep_running = 1;
   while( keep_running ) {       // you're main loop here... 
      // if you use select(), see snmp_select_info() in snmp_api(3) 
      //     --- OR ---  
      agent_check_and_process( 1 ); // 0 == don't block 
   }
   snmp_shutdown( daemon_title );   // завершение работы
   SOCK_CLEANUP;
   if( debug_level != 0 ) snmp_log( LOG_INFO, "%s was finished.\n", daemon_title );
   return EXIT_SUCCESS;
}
